#include <stdio.h>
#include <stdlib.h>
#include "base.h"
#include "maxmin.h"
#include "permute.h"

typedef struct Perm_T *Perm_T;
struct Perm_T {
  int position;
  double mean;
};

static Perm_T
Perm_new (int position, double mean) {
  Perm_T new = (Perm_T) malloc(sizeof(*new));

  new->position = position;
  new->mean = mean;
  return new;
}

static void
Perm_free (Perm_T *perm) {
  free(*perm);
}


static int
compare_perms (const void *x, const void *y) {
  Perm_T perm1 = * (Perm_T *) x;
  Perm_T perm2 = * (Perm_T *) y;

  /* printf("Comparing %f with %f\n",perm1->mean,perm2->mean); */
  if (perm1->mean < perm2->mean) {
    return -1;
  } else if (perm1->mean >= perm2->mean) {
    return 1;
  } else {
    return 0;
  }
}

static double
mean_column (int **profile, int position, Frequencies_T frequencies) {
  int aa;
  double prob, mean = 0.0;

  for (aa = 0; aa < NAT_AAS; ++aa) {
    prob = Frequencies_get(frequencies,aa);
    mean += prob * (double) profile[position][aa];
  }
  return mean;
}

void
Permute_print_ordering (int *ordering, int width) {
  int j;

  for (j = 0; j < width; ++j) {
    printf("%d ",ordering[j]);
  }
  printf("\n");
}


int *
Permute_find_ordering (int **profile, int width, Frequencies_T frequencies) {
  int *ordering, j;
  double mean;
  Perm_T *perms;

  ordering = (int *) calloc(width,sizeof(int));
  perms = (Perm_T *) calloc(width,sizeof(Perm_T));
  for (j = 0; j < width; ++j) {
    mean = mean_column(profile,j,frequencies);
    perms[j] = Perm_new(j,mean-(double) max_column(profile,j));
  }
  qsort(perms,width,sizeof(Perm_T),compare_perms);
  for (j = 0; j < width; ++j) {
    ordering[j] = perms[j]->position;
  }
  for (j = 0; j < width; ++j) {
    Perm_free(&(perms[j]));
  }
  free(perms);
  return ordering;
}

